/*
 * ASUS K501UQ - OpenCore hotpatches
 * Battery compatibility patches
 *
 * For information on building this patch refer to 
 * https://www.tonymacx86.com/threads/guide-using-clover-to-hotpatch-acpi.200137/post-1308261
 * 
 * Required renames:
 * Replacement methods
 *      5F42495800 => 5842495800 _BIX => XBIX
 *      4249464100 => 5849464100 BIFA => XIFA
 *      534D42520B => 584D42520B SMBR => XMBR
 *      534D42570D => 584D42570D SMBW => XMBW
 *      4543534207 => 5843534207 ECSB => XCSB
 *      5441434809 => 5841434809 TACH => XACH
 * FBST - fix battery status at full charge
 *      43484753 00700A02 60A10470 0160 =>
 *      43484753 00700A02 60A10470 0060

 */
DefinitionBlock ("", "SSDT", 2, "hack", "BATT", 0x00000000)
{
    // Here's a LOT of external declarations
    External (_SB_.PCI0, DeviceObj)
    External (_SB_.PCI0.BAT0, DeviceObj)
    External (_SB_.PCI0.BAT0._BIF, MethodObj)    // 0 Arguments
    External (_SB_.PCI0.BAT0.BIXT, PkgObj)
    External (_SB_.PCI0.BAT0.NBIX, PkgObj)
    External (_SB_.PCI0.BAT0.PBIF, PkgObj)
    External (_SB_.PCI0.LPCB, DeviceObj)
    External (_SB_.PCI0.LPCB.EC0_, DeviceObj)
    External (_SB_.PCI0.LPCB.EC0_.ADD2, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.ADDR, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.BATP, MethodObj)    // 1 Arguments
    External (_SB_.PCI0.LPCB.EC0_.BCN2, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.BCNT, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.CMD2, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.CMDB, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.DA20, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.DA21, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.DAT0, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.DAT1, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.ECAV, MethodObj)    // 0 Arguments
    External (_SB_.PCI0.LPCB.EC0_.GBTT, MethodObj)    // 1 Arguments
    External (_SB_.PCI0.LPCB.EC0_.MUEC, MutexObj)
    External (_SB_.PCI0.LPCB.EC0_.PRT2, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.PRTC, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.RCBT, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.RDBL, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.RDBT, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.RDQK, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.RDWD, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.SBBY, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.SDBT, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.SST2, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.SSTS, FieldUnitObj)
    External (_SB_.PCI0.LPCB.EC0_.SWTC, MethodObj)    // 1 Arguments
    External (_SB_.PCI0.LPCB.EC0_.WRBL, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.WRBT, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.WRQK, IntObj)
    External (_SB_.PCI0.LPCB.EC0_.WRWD, IntObj)
    External (BCN2, IntObj)
    External (BCNT, IntObj)
    External (BIXT, IntObj)
    External (BSLF, IntObj)
    External (DA20, IntObj)
    External (DA21, IntObj)
    External (DAT0, IntObj)
    External (DAT1, IntObj)
    External (NBIX, IntObj)
    External (PRT2, IntObj)
    External (PRTC, IntObj)
    External (SBBY, IntObj)
    External (SST2, IntObj)
    External (SSTS, IntObj)

    // Utility method for accessing 16-byte fields
    Method (B1B2, 2, NotSerialized)
    {
        Return (Or (Arg0, ShiftLeft (Arg1, 0x08)))
    }

    Scope (_SB.PCI0)
    {
        Scope (BAT0)
        {
            // Replacement method
            Method (_BIX, 0, NotSerialized)  // _BIX: Battery Information Extended
            {
                If (LNot (^^LPCB.EC0.BATP (Zero)))
                {
                    Return (NBIX) /* External reference */
                }

                If (LEqual (^^LPCB.EC0.GBTT (Zero), 0xFF))
                {
                    Return (NBIX) /* External reference */
                }

                _BIF ()
                Store (DerefOf (Index (PBIF, Zero)), Index (BIXT, One))
                Store (DerefOf (Index (PBIF, One)), Index (BIXT, 0x02))
                Store (DerefOf (Index (PBIF, 0x02)), Index (BIXT, 0x03))
                Store (DerefOf (Index (PBIF, 0x03)), Index (BIXT, 0x04))
                Store (DerefOf (Index (PBIF, 0x04)), Index (BIXT, 0x05))
                Store (DerefOf (Index (PBIF, 0x05)), Index (BIXT, 0x06))
                Store (DerefOf (Index (PBIF, 0x06)), Index (BIXT, 0x07))
                Store (DerefOf (Index (PBIF, 0x07)), Index (BIXT, 0x0E))
                Store (DerefOf (Index (PBIF, 0x08)), Index (BIXT, 0x0F))
                Store (DerefOf (Index (PBIF, 0x09)), Index (BIXT, 0x10))
                Store (DerefOf (Index (PBIF, 0x0A)), Index (BIXT, 0x11))
                Store (DerefOf (Index (PBIF, 0x0B)), Index (BIXT, 0x12))
                Store (DerefOf (Index (PBIF, 0x0C)), Index (BIXT, 0x13))
                If (LEqual (DerefOf (Index (BIXT, One)), One))
                {
                    Store (Zero, Index (BIXT, One))
                    Store (DerefOf (Index (BIXT, 0x05)), Local0)
                    Multiply (DerefOf (Index (BIXT, 0x02)), Local0, Index (BIXT, 0x02))
                    Multiply (DerefOf (Index (BIXT, 0x03)), Local0, Index (BIXT, 0x03))
                    Multiply (DerefOf (Index (BIXT, 0x06)), Local0, Index (BIXT, 0x06))
                    Multiply (DerefOf (Index (BIXT, 0x07)), Local0, Index (BIXT, 0x07))
                    Multiply (DerefOf (Index (BIXT, 0x0E)), Local0, Index (BIXT, 0x0E))
                    Multiply (DerefOf (Index (BIXT, 0x0F)), Local0, Index (BIXT, 0x0F))
                    Divide (DerefOf (Index (BIXT, 0x02)), 0x03E8, Local0, Index (BIXT, 0x02))
                    Divide (DerefOf (Index (BIXT, 0x03)), 0x03E8, Local0, Index (BIXT, 0x03))
                    Divide (DerefOf (Index (BIXT, 0x06)), 0x03E8, Local0, Index (BIXT, 0x06))
                    Divide (DerefOf (Index (BIXT, 0x07)), 0x03E8, Local0, Index (BIXT, 0x07))
                    Divide (DerefOf (Index (BIXT, 0x0E)), 0x03E8, Local0, Index (BIXT, 0x0E))
                    Divide (DerefOf (Index (BIXT, 0x0F)), 0x03E8, Local0, Index (BIXT, 0x0F))
                }

                Store (B1B2 (^^LPCB.EC0.XC30, ^^LPCB.EC0.XC31), Index (BIXT, 0x08))
                Store (0x0001869F, Index (BIXT, 0x09))
                Return (BIXT) /* External reference */
            }
        }

        Scope (LPCB.EC0)
        {
            // EC overlays
            OperationRegion (XCOR, EmbeddedControl, Zero, 0xFF)
            Field (XCOR, ByteAcc, Lock, Preserve)
            {
                Offset (0x93), 
                TH00,   8, // TAH0
                TH01,   8, 
                TH10,   8, // TAH1
                TH11,   8, 
                Offset (0xC4), 
                XC30,   8, // B0C3
                XC31,   8, 
                Offset (0xF4), 
                B0S0,   8, // B0SN
                B0S1,   8, 
                Offset (0xFC), 
                B1S0,   8, // B1SN
                B1S1,   8
            }

            OperationRegion (XMBX, EmbeddedControl, 0x18, 0x28)
            Field (XMBX, ByteAcc, NoLock, Preserve)
            {
                Offset (0x04), 
                BA00,   8, //BDAT, 256
                BA01,   8, 
                BA02,   8, 
                BA03,   8, 
                BA04,   8, 
                BA05,   8, 
                BA06,   8, 
                BA07,   8, 
                BA08,   8, 
                BA09,   8, 
                BA0A,   8, 
                BA0B,   8, 
                BA0C,   8, 
                BA0D,   8, 
                BA0E,   8, 
                BA0F,   8, 
                BA10,   8, 
                BA11,   8, 
                BA12,   8, 
                BA13,   8, 
                BA14,   8, 
                BA15,   8, 
                BA16,   8, 
                BA17,   8, 
                BA18,   8, 
                BA19,   8, 
                BA1A,   8, 
                BA1B,   8, 
                BA1C,   8, 
                BA1D,   8, 
                BA1E,   8, 
                BA1F,   8
            }

            OperationRegion (XMB2, EmbeddedControl, 0x40, 0x28)
            Field (XMB2, ByteAcc, NoLock, Preserve)
            {
                Offset (0x04), 
                BB00,   8, //BDA2, 256
                BB01,   8, 
                BB02,   8, 
                BB03,   8, 
                BB04,   8, 
                BB05,   8, 
                BB06,   8, 
                BB07,   8, 
                BB08,   8, 
                BB09,   8, 
                BB0A,   8, 
                BB0B,   8, 
                BB0C,   8, 
                BB0D,   8, 
                BB0E,   8, 
                BB0F,   8, 
                BB10,   8, 
                BB11,   8, 
                BB12,   8, 
                BB13,   8, 
                BB14,   8, 
                BB15,   8, 
                BB16,   8, 
                BB17,   8, 
                BB18,   8, 
                BB19,   8, 
                BB1A,   8, 
                BB1B,   8, 
                BB1C,   8, 
                BB1D,   8, 
                BB1E,   8, 
                BB1F,   8
            }

            Field (XMBX, ByteAcc, NoLock, Preserve)
            {
                Offset (0x04), 
                T2B0,   8, // DT2B
                T2B1,   8
            }

            // Replacement methods
            Method (BIFA, 0, NotSerialized)
            {
                If (ECAV ())
                {
                    If (BSLF)
                    {
                        Store (B1B2 (B1S0, B1S1), Local0)
                    }
                    Else
                    {
                        Store (B1B2 (B0S0, B0S1), Local0)
                    }
                }
                Else
                {
                    Store (Ones, Local0)
                }

                Return (Local0)
            }

            Method (SMBR, 3, Serialized)
            {
                Store (Package (0x03)
                    {
                        0x07, 
                        Zero, 
                        Zero
                    }, Local0)
                If (LNot (ECAV ()))
                {
                    Return (Local0)
                }

                If (LNotEqual (Arg0, RDBL))
                {
                    If (LNotEqual (Arg0, RDWD))
                    {
                        If (LNotEqual (Arg0, RDBT))
                        {
                            If (LNotEqual (Arg0, RCBT))
                            {
                                If (LNotEqual (Arg0, RDQK))
                                {
                                    Return (Local0)
                                }
                            }
                        }
                    }
                }

                Acquire (MUEC, 0xFFFF)
                Store (PRTC, Local1)
                Store (Zero, Local2)
                While (LNotEqual (Local1, Zero))
                {
                    Stall (0x0A)
                    Increment (Local2)
                    If (LGreater (Local2, 0x03E8))
                    {
                        Store (SBBY, Index (Local0, Zero))
                        Store (Zero, Local1)
                    }
                    Else
                    {
                        Store (PRTC, Local1)
                    }
                }

                If (LLessEqual (Local2, 0x03E8))
                {
                    ShiftLeft (Arg1, One, Local3)
                    Or (Local3, One, Local3)
                    Store (Local3, ADDR) /* External reference */
                    If (LNotEqual (Arg0, RDQK))
                    {
                        If (LNotEqual (Arg0, RCBT))
                        {
                            Store (Arg2, CMDB) /* External reference */
                        }
                    }

                    WRBA (Zero)
                    Store (Arg0, PRTC) /* External reference */
                    Store (SWTC (Arg0), Index (Local0, Zero))
                    If (LEqual (DerefOf (Index (Local0, Zero)), Zero))
                    {
                        If (LEqual (Arg0, RDBL))
                        {
                            Store (BCNT, Index (Local0, One))
                            Store (RDBA (), Index (Local0, 0x02))
                        }

                        If (LEqual (Arg0, RDWD))
                        {
                            Store (0x02, Index (Local0, One))
                            Store (B1B2 (T2B0, T2B1), Index (Local0, 0x02))
                        }

                        If (LEqual (Arg0, RDBT))
                        {
                            Store (One, Index (Local0, One))
                            Store (DAT0, Index (Local0, 0x02))
                        }

                        If (LEqual (Arg0, RCBT))
                        {
                            Store (One, Index (Local0, One))
                            Store (DAT0, Index (Local0, 0x02))
                        }
                    }
                }

                Release (MUEC)
                Return (Local0)
            }

            Method (SMBW, 5, Serialized)
            {
                Store (Package (0x01)
                    {
                        0x07
                    }, Local0)
                If (LNot (ECAV ()))
                {
                    Return (Local0)
                }

                If (LNotEqual (Arg0, WRBL))
                {
                    If (LNotEqual (Arg0, WRWD))
                    {
                        If (LNotEqual (Arg0, WRBT))
                        {
                            If (LNotEqual (Arg0, SDBT))
                            {
                                If (LNotEqual (Arg0, WRQK))
                                {
                                    Return (Local0)
                                }
                            }
                        }
                    }
                }

                Acquire (MUEC, 0xFFFF)
                Store (PRTC, Local1)
                Store (Zero, Local2)
                While (LNotEqual (Local1, Zero))
                {
                    Stall (0x0A)
                    Increment (Local2)
                    If (LGreater (Local2, 0x03E8))
                    {
                        Store (SBBY, Index (Local0, Zero))
                        Store (Zero, Local1)
                    }
                    Else
                    {
                        Store (PRTC, Local1)
                    }
                }

                If (LLessEqual (Local2, 0x03E8))
                {
                    WRBA (Zero)
                    ShiftLeft (Arg1, One, Local3)
                    Store (Local3, ADDR) /* External reference */
                    If (LNotEqual (Arg0, WRQK))
                    {
                        If (LNotEqual (Arg0, SDBT))
                        {
                            Store (Arg2, CMDB) /* External reference */
                        }
                    }

                    If (LEqual (Arg0, WRBL))
                    {
                        Store (Arg3, BCNT) /* External reference */
                        WRBA (Arg4)
                    }

                    If (LEqual (Arg0, WRWD))
                    {
                        Store (Arg4, T2B0) /* \_SB_.PCI0.LPCB.EC0_.T2B0 */
                        Store (ShiftRight (Arg4, 0x08), T2B1) /* \_SB_.PCI0.LPCB.EC0_.T2B1 */
                    }

                    If (LEqual (Arg0, WRBT))
                    {
                        Store (Arg4, DAT0) /* External reference */
                    }

                    If (LEqual (Arg0, SDBT))
                    {
                        Store (Arg4, DAT0) /* External reference */
                    }

                    Store (Arg0, PRTC) /* External reference */
                    Store (SWTC (Arg0), Index (Local0, Zero))
                }

                Release (MUEC)
                Return (Local0)
            }

            Method (ECSB, 7, NotSerialized)
            {
                Store (Package (0x05)
                    {
                        0x11, 
                        Zero, 
                        Zero, 
                        Zero, 
                        Buffer (0x20){}
                    }, Local1)
                If (LGreater (Arg0, One))
                {
                    Return (Local1)
                }

                If (ECAV ())
                {
                    Acquire (MUEC, 0xFFFF)
                    If (LEqual (Arg0, Zero))
                    {
                        Store (PRTC, Local0)
                    }
                    Else
                    {
                        Store (PRT2, Local0)
                    }

                    Store (Zero, Local2)
                    While (LNotEqual (Local0, Zero))
                    {
                        Stall (0x0A)
                        Increment (Local2)
                        If (LGreater (Local2, 0x03E8))
                        {
                            Store (SBBY, Index (Local1, Zero))
                            Store (Zero, Local0)
                        }
                        ElseIf (LEqual (Arg0, Zero))
                        {
                            Store (PRTC, Local0)
                        }
                        Else
                        {
                            Store (PRT2, Local0)
                        }
                    }

                    If (LLessEqual (Local2, 0x03E8))
                    {
                        If (LEqual (Arg0, Zero))
                        {
                            Store (Arg2, ADDR) /* External reference */
                            Store (Arg3, CMDB) /* External reference */
                            If (LOr (LEqual (Arg1, 0x0A), LEqual (Arg1, 0x0B)))
                            {
                                Store (DerefOf (Index (Arg6, Zero)), BCNT) /* External reference */
                                WRBA (DerefOf (Index (Arg6, One)))
                            }
                            Else
                            {
                                Store (Arg4, DAT0) /* External reference */
                                Store (Arg5, DAT1) /* External reference */
                            }

                            Store (Arg1, PRTC) /* External reference */
                        }
                        Else
                        {
                            Store (Arg2, ADD2) /* External reference */
                            Store (Arg3, CMD2) /* External reference */
                            If (LOr (LEqual (Arg1, 0x0A), LEqual (Arg1, 0x0B)))
                            {
                                Store (DerefOf (Index (Arg6, Zero)), BCN2) /* External reference */
                                WRBB (DerefOf (Index (Arg6, One)))
                            }
                            Else
                            {
                                Store (Arg4, DA20) /* External reference */
                                Store (Arg5, DA21) /* External reference */
                            }

                            Store (Arg1, PRT2) /* External reference */
                        }

                        Store (0x7F, Local0)
                        If (LEqual (Arg0, Zero))
                        {
                            While (PRTC)
                            {
                                Sleep (One)
                                Decrement (Local0)
                            }
                        }
                        Else
                        {
                            While (PRT2)
                            {
                                Sleep (One)
                                Decrement (Local0)
                            }
                        }

                        If (Local0)
                        {
                            If (LEqual (Arg0, Zero))
                            {
                                Store (SSTS, Local0)
                                Store (DAT0, Index (Local1, One))
                                Store (DAT1, Index (Local1, 0x02))
                                Store (BCNT, Index (Local1, 0x03))
                                Store (RDBA (), Index (Local1, 0x04))
                            }
                            Else
                            {
                                Store (SST2, Local0)
                                Store (DA20, Index (Local1, One))
                                Store (DA21, Index (Local1, 0x02))
                                Store (BCN2, Index (Local1, 0x03))
                                Store (RDBB (), Index (Local1, 0x04))
                            }

                            And (Local0, 0x1F, Local0)
                            If (Local0)
                            {
                                Add (Local0, 0x10, Local0)
                            }

                            Store (Local0, Index (Local1, Zero))
                        }
                        Else
                        {
                            Store (0x10, Index (Local1, Zero))
                        }
                    }

                    Release (MUEC)
                }

                Return (Local1)
            }

            Method (TACH, 1, Serialized)
            {
                If (ECAV ())
                {
                    Switch (Arg0)
                    {
                        Case (Zero)
                        {
                            Store (B1B2 (TH00, TH01), Local0)
                            Break
                        }
                        Case (One)
                        {
                            Store (B1B2 (TH10, TH11), Local0)
                            Break
                        }
                        Default
                        {
                            Return (Ones)
                        }

                    }

                    Multiply (Local0, 0x02, Local0)
                    If (LNotEqual (Local0, Zero))
                    {
                        Divide (0x0041CDB4, Local0, Local1, Local0)
                        Return (Local0)
                    }
                    Else
                    {
                        Return (Ones)
                    }
                }
                Else
                {
                    Return (Ones)
                }
            }

            // Utility methods for accessing large fields
            Method (RDBA, 0, Serialized)
            {
                Name (TEMP, Buffer (0x20){})
                Store (BA00, Index (TEMP, Zero))
                Store (BA01, Index (TEMP, One))
                Store (BA02, Index (TEMP, 0x02))
                Store (BA03, Index (TEMP, 0x03))
                Store (BA04, Index (TEMP, 0x04))
                Store (BA05, Index (TEMP, 0x05))
                Store (BA06, Index (TEMP, 0x06))
                Store (BA07, Index (TEMP, 0x07))
                Store (BA08, Index (TEMP, 0x08))
                Store (BA09, Index (TEMP, 0x09))
                Store (BA0A, Index (TEMP, 0x0A))
                Store (BA0B, Index (TEMP, 0x0B))
                Store (BA0C, Index (TEMP, 0x0C))
                Store (BA0D, Index (TEMP, 0x0D))
                Store (BA0E, Index (TEMP, 0x0E))
                Store (BA0F, Index (TEMP, 0x0F))
                Store (BA10, Index (TEMP, 0x10))
                Store (BA11, Index (TEMP, 0x11))
                Store (BA12, Index (TEMP, 0x12))
                Store (BA13, Index (TEMP, 0x13))
                Store (BA14, Index (TEMP, 0x14))
                Store (BA15, Index (TEMP, 0x15))
                Store (BA16, Index (TEMP, 0x16))
                Store (BA17, Index (TEMP, 0x17))
                Store (BA18, Index (TEMP, 0x18))
                Store (BA19, Index (TEMP, 0x19))
                Store (BA1A, Index (TEMP, 0x1A))
                Store (BA1B, Index (TEMP, 0x1B))
                Store (BA1C, Index (TEMP, 0x1C))
                Store (BA1D, Index (TEMP, 0x1D))
                Store (BA1E, Index (TEMP, 0x1E))
                Store (BA1F, Index (TEMP, 0x1F))
                Return (TEMP) /* \_SB_.PCI0.LPCB.EC0_.RDBA.TEMP */
            }

            Method (WRBA, 1, Serialized)
            {
                Name (TEMP, Buffer (0x20){})
                Store (Arg0, TEMP) /* \_SB_.PCI0.LPCB.EC0_.WRBA.TEMP */
                Store (DerefOf (Index (TEMP, Zero)), BA00) /* \_SB_.PCI0.LPCB.EC0_.BA00 */
                Store (DerefOf (Index (TEMP, One)), BA01) /* \_SB_.PCI0.LPCB.EC0_.BA01 */
                Store (DerefOf (Index (TEMP, 0x02)), BA02) /* \_SB_.PCI0.LPCB.EC0_.BA02 */
                Store (DerefOf (Index (TEMP, 0x03)), BA03) /* \_SB_.PCI0.LPCB.EC0_.BA03 */
                Store (DerefOf (Index (TEMP, 0x04)), BA04) /* \_SB_.PCI0.LPCB.EC0_.BA04 */
                Store (DerefOf (Index (TEMP, 0x05)), BA05) /* \_SB_.PCI0.LPCB.EC0_.BA05 */
                Store (DerefOf (Index (TEMP, 0x06)), BA06) /* \_SB_.PCI0.LPCB.EC0_.BA06 */
                Store (DerefOf (Index (TEMP, 0x07)), BA07) /* \_SB_.PCI0.LPCB.EC0_.BA07 */
                Store (DerefOf (Index (TEMP, 0x08)), BA08) /* \_SB_.PCI0.LPCB.EC0_.BA08 */
                Store (DerefOf (Index (TEMP, 0x09)), BA09) /* \_SB_.PCI0.LPCB.EC0_.BA09 */
                Store (DerefOf (Index (TEMP, 0x0A)), BA0A) /* \_SB_.PCI0.LPCB.EC0_.BA0A */
                Store (DerefOf (Index (TEMP, 0x0B)), BA0B) /* \_SB_.PCI0.LPCB.EC0_.BA0B */
                Store (DerefOf (Index (TEMP, 0x0C)), BA0C) /* \_SB_.PCI0.LPCB.EC0_.BA0C */
                Store (DerefOf (Index (TEMP, 0x0D)), BA0D) /* \_SB_.PCI0.LPCB.EC0_.BA0D */
                Store (DerefOf (Index (TEMP, 0x0E)), BA0E) /* \_SB_.PCI0.LPCB.EC0_.BA0E */
                Store (DerefOf (Index (TEMP, 0x0F)), BA0F) /* \_SB_.PCI0.LPCB.EC0_.BA0F */
                Store (DerefOf (Index (TEMP, 0x10)), BA10) /* \_SB_.PCI0.LPCB.EC0_.BA10 */
                Store (DerefOf (Index (TEMP, 0x11)), BA11) /* \_SB_.PCI0.LPCB.EC0_.BA11 */
                Store (DerefOf (Index (TEMP, 0x12)), BA12) /* \_SB_.PCI0.LPCB.EC0_.BA12 */
                Store (DerefOf (Index (TEMP, 0x13)), BA13) /* \_SB_.PCI0.LPCB.EC0_.BA13 */
                Store (DerefOf (Index (TEMP, 0x14)), BA14) /* \_SB_.PCI0.LPCB.EC0_.BA14 */
                Store (DerefOf (Index (TEMP, 0x15)), BA15) /* \_SB_.PCI0.LPCB.EC0_.BA15 */
                Store (DerefOf (Index (TEMP, 0x16)), BA16) /* \_SB_.PCI0.LPCB.EC0_.BA16 */
                Store (DerefOf (Index (TEMP, 0x17)), BA17) /* \_SB_.PCI0.LPCB.EC0_.BA17 */
                Store (DerefOf (Index (TEMP, 0x18)), BA18) /* \_SB_.PCI0.LPCB.EC0_.BA18 */
                Store (DerefOf (Index (TEMP, 0x19)), BA19) /* \_SB_.PCI0.LPCB.EC0_.BA19 */
                Store (DerefOf (Index (TEMP, 0x1A)), BA1A) /* \_SB_.PCI0.LPCB.EC0_.BA1A */
                Store (DerefOf (Index (TEMP, 0x1B)), BA1B) /* \_SB_.PCI0.LPCB.EC0_.BA1B */
                Store (DerefOf (Index (TEMP, 0x1C)), BA1C) /* \_SB_.PCI0.LPCB.EC0_.BA1C */
                Store (DerefOf (Index (TEMP, 0x1D)), BA1D) /* \_SB_.PCI0.LPCB.EC0_.BA1D */
                Store (DerefOf (Index (TEMP, 0x1E)), BA1E) /* \_SB_.PCI0.LPCB.EC0_.BA1E */
                Store (DerefOf (Index (TEMP, 0x1F)), BA1F) /* \_SB_.PCI0.LPCB.EC0_.BA1F */
            }

            Method (RDBB, 0, Serialized)
            {
                Name (TEMP, Buffer (0x20){})
                Store (BB00, Index (TEMP, Zero))
                Store (BB01, Index (TEMP, One))
                Store (BB02, Index (TEMP, 0x02))
                Store (BB03, Index (TEMP, 0x03))
                Store (BB04, Index (TEMP, 0x04))
                Store (BB05, Index (TEMP, 0x05))
                Store (BB06, Index (TEMP, 0x06))
                Store (BB07, Index (TEMP, 0x07))
                Store (BB08, Index (TEMP, 0x08))
                Store (BB09, Index (TEMP, 0x09))
                Store (BB0A, Index (TEMP, 0x0A))
                Store (BB0B, Index (TEMP, 0x0B))
                Store (BB0C, Index (TEMP, 0x0C))
                Store (BB0D, Index (TEMP, 0x0D))
                Store (BB0E, Index (TEMP, 0x0E))
                Store (BB0F, Index (TEMP, 0x0F))
                Store (BB10, Index (TEMP, 0x10))
                Store (BB11, Index (TEMP, 0x11))
                Store (BB12, Index (TEMP, 0x12))
                Store (BB13, Index (TEMP, 0x13))
                Store (BB14, Index (TEMP, 0x14))
                Store (BB15, Index (TEMP, 0x15))
                Store (BB16, Index (TEMP, 0x16))
                Store (BB17, Index (TEMP, 0x17))
                Store (BB18, Index (TEMP, 0x18))
                Store (BB19, Index (TEMP, 0x19))
                Store (BB1A, Index (TEMP, 0x1A))
                Store (BB1B, Index (TEMP, 0x1B))
                Store (BB1C, Index (TEMP, 0x1C))
                Store (BB1D, Index (TEMP, 0x1D))
                Store (BB1E, Index (TEMP, 0x1E))
                Store (BB1F, Index (TEMP, 0x1F))
                Return (TEMP) /* \_SB_.PCI0.LPCB.EC0_.RDBB.TEMP */
            }

            Method (WRBB, 1, Serialized)
            {
                Name (TEMP, Buffer (0x20){})
                Store (Arg0, TEMP) /* \_SB_.PCI0.LPCB.EC0_.WRBB.TEMP */
                Store (DerefOf (Index (TEMP, Zero)), BB00) /* \_SB_.PCI0.LPCB.EC0_.BB00 */
                Store (DerefOf (Index (TEMP, One)), BB01) /* \_SB_.PCI0.LPCB.EC0_.BB01 */
                Store (DerefOf (Index (TEMP, 0x02)), BB02) /* \_SB_.PCI0.LPCB.EC0_.BB02 */
                Store (DerefOf (Index (TEMP, 0x03)), BB03) /* \_SB_.PCI0.LPCB.EC0_.BB03 */
                Store (DerefOf (Index (TEMP, 0x04)), BB04) /* \_SB_.PCI0.LPCB.EC0_.BB04 */
                Store (DerefOf (Index (TEMP, 0x05)), BB05) /* \_SB_.PCI0.LPCB.EC0_.BB05 */
                Store (DerefOf (Index (TEMP, 0x06)), BB06) /* \_SB_.PCI0.LPCB.EC0_.BB06 */
                Store (DerefOf (Index (TEMP, 0x07)), BB07) /* \_SB_.PCI0.LPCB.EC0_.BB07 */
                Store (DerefOf (Index (TEMP, 0x08)), BB08) /* \_SB_.PCI0.LPCB.EC0_.BB08 */
                Store (DerefOf (Index (TEMP, 0x09)), BB09) /* \_SB_.PCI0.LPCB.EC0_.BB09 */
                Store (DerefOf (Index (TEMP, 0x0A)), BB0A) /* \_SB_.PCI0.LPCB.EC0_.BB0A */
                Store (DerefOf (Index (TEMP, 0x0B)), BB0B) /* \_SB_.PCI0.LPCB.EC0_.BB0B */
                Store (DerefOf (Index (TEMP, 0x0C)), BB0C) /* \_SB_.PCI0.LPCB.EC0_.BB0C */
                Store (DerefOf (Index (TEMP, 0x0D)), BB0D) /* \_SB_.PCI0.LPCB.EC0_.BB0D */
                Store (DerefOf (Index (TEMP, 0x0E)), BB0E) /* \_SB_.PCI0.LPCB.EC0_.BB0E */
                Store (DerefOf (Index (TEMP, 0x0F)), BB0F) /* \_SB_.PCI0.LPCB.EC0_.BB0F */
                Store (DerefOf (Index (TEMP, 0x10)), BB10) /* \_SB_.PCI0.LPCB.EC0_.BB10 */
                Store (DerefOf (Index (TEMP, 0x11)), BB11) /* \_SB_.PCI0.LPCB.EC0_.BB11 */
                Store (DerefOf (Index (TEMP, 0x12)), BB12) /* \_SB_.PCI0.LPCB.EC0_.BB12 */
                Store (DerefOf (Index (TEMP, 0x13)), BB13) /* \_SB_.PCI0.LPCB.EC0_.BB13 */
                Store (DerefOf (Index (TEMP, 0x14)), BB14) /* \_SB_.PCI0.LPCB.EC0_.BB14 */
                Store (DerefOf (Index (TEMP, 0x15)), BB15) /* \_SB_.PCI0.LPCB.EC0_.BB15 */
                Store (DerefOf (Index (TEMP, 0x16)), BB16) /* \_SB_.PCI0.LPCB.EC0_.BB16 */
                Store (DerefOf (Index (TEMP, 0x17)), BB17) /* \_SB_.PCI0.LPCB.EC0_.BB17 */
                Store (DerefOf (Index (TEMP, 0x18)), BB18) /* \_SB_.PCI0.LPCB.EC0_.BB18 */
                Store (DerefOf (Index (TEMP, 0x19)), BB19) /* \_SB_.PCI0.LPCB.EC0_.BB19 */
                Store (DerefOf (Index (TEMP, 0x1A)), BB1A) /* \_SB_.PCI0.LPCB.EC0_.BB1A */
                Store (DerefOf (Index (TEMP, 0x1B)), BB1B) /* \_SB_.PCI0.LPCB.EC0_.BB1B */
                Store (DerefOf (Index (TEMP, 0x1C)), BB1C) /* \_SB_.PCI0.LPCB.EC0_.BB1C */
                Store (DerefOf (Index (TEMP, 0x1D)), BB1D) /* \_SB_.PCI0.LPCB.EC0_.BB1D */
                Store (DerefOf (Index (TEMP, 0x1E)), BB1E) /* \_SB_.PCI0.LPCB.EC0_.BB1E */
                Store (DerefOf (Index (TEMP, 0x1F)), BB1F) /* \_SB_.PCI0.LPCB.EC0_.BB1F */
            }
        }
    }
}

