/*
 * ASUS K501UQ - OpenCore hotpatches
 * NVidia dGPU disabler
 * 
 * Explanations:
 * https://www.tonymacx86.com/threads/guide-disabling-discrete-graphics-in-dual-gpu-laptops.163772/
 * 
 * Required patches:
 * DSDT only
 * _REG => XREG
 *     5F524547 02A00B93 680A0370 69454346 4C =>
 *     58524547 02A00B93 680A0370 69454346 4C
 * _WAK => XWAK
 *     5F57414B015257414B  =>  5857414B015257414B
 * _PTS => XPTS
 *     5F50545301  =>  5850545301
 * SSDT-5 only (OemTableId=53675063 68000000): 
 * _INI => XINI
 *     5F494E49 0070005C 2F055F53 425F5043 49305250 30315045 4750  =>  
 *     58494E49 0070005C 2F055F53 425F5043 49305250 30315045 4750
 * HGOF => XGOF
 *     48474F4608A00A92  =>  58474F4608A00A92
 */
DefinitionBlock ("", "SSDT", 2, "hack", "DGPU", 0x00000000)
{
    External (_SB_.PCI0.LPCB.EC0_, DeviceObj)
    External (_SB_.PCI0.LPCB.EC0_.RRAM, MethodObj)    // 1 Arguments
    External (_SB_.PCI0.LPCB.EC0_.WRAM, MethodObj)    // 2 Arguments
    External (_SB_.PCI0.LPCB.EC0_.XREG, MethodObj)    // 2 Arguments
    External (_SB_.PCI0.RP01.PEGP, DeviceObj)
    External (_SB_.PCI0.RP01.PEGP._OFF, MethodObj)    // 0 Arguments
    External (_SB_.PCI0.RP01.PEGP._ON_, MethodObj)    // 0 Arguments
    External (_SB_.PCI0.RP01.PEGP.XGOF, MethodObj)    // 0 Arguments
    External (_SB_.PCI0.RP01.PEGP.XINI, MethodObj)    // 0 Arguments
    External (XPTS, MethodObj)    // 1 Arguments
    External (XWAK, MethodObj)    // 1 Arguments

    Scope (\_SB.PCI0.RP01.PEGP)
    {
        // Disabling dGPU right at init.
        Method (_INI, 0, NotSerialized)  // _INI: Initialize
        {
            XINI ()
            If (_OSI ("Darwin"))
            {
                _OFF ()
            }
        }

        // Disabling some EC stuff that cannot be called from _OFF (and it calls HGOF).
        // Fortunately we can call all that stuff at the end of this method.
        Method (HGOF, 0, Serialized)
        {
            XGOF ()
            If (Not (_OSI ("Darwin")))
            {
                Store (\_SB.PCI0.LPCB.EC0.RRAM (0xF921), Local0)
                And (Local0, 0xCF, Local0)
                \_SB.PCI0.LPCB.EC0.WRAM (0xF921, Local0)
                \_SB.PCI0.LPCB.EC0.WRAM (0xF920, 0x95)
                \_SB.PCI0.LPCB.EC0.WRAM (0xF7A4, Zero)
                \_SB.PCI0.LPCB.EC0.WRAM (0xF7A5, Zero)
            }

            Return (Zero)
        }
    }

    Scope (\_SB.PCI0.LPCB.EC0)
    {
        // Calling _OFF from _REG.
        Method (_REG, 2, NotSerialized)  // _REG: Region Availability
        {
            XREG (Arg0, Arg1)
            If (_OSI ("Darwin"))
            {
                If (LAnd (LEqual (Arg0, 0x03), LEqual (Arg1, One)))
                {
                    ^^^RP01.PEGP._OFF ()
                }
            }
        }
    }

    // Calling _OFF and running that disabled EC stuff from _WAK to disable it again at wake.
    Method (_WAK, 1, NotSerialized)  // _WAK: Wake
    {
        Store (XWAK (Arg0), Local0)
        If (_OSI ("Darwin"))
        {
            \_SB.PCI0.RP01.PEGP._OFF ()
            Store (\_SB.PCI0.LPCB.EC0.RRAM (0xF921), Local1)
            And (Local1, 0xCF, Local1)
            \_SB.PCI0.LPCB.EC0.WRAM (0xF921, Local1)
            \_SB.PCI0.LPCB.EC0.WRAM (0xF920, 0x95)
            \_SB.PCI0.LPCB.EC0.WRAM (0xF7A4, Zero)
            \_SB.PCI0.LPCB.EC0.WRAM (0xF7A5, Zero)
        }

        Return (Local0)
    }

    // Re-enabling dGPU at sleep.
    Method (_PTS, 1, NotSerialized)  // _PTS: Prepare To Sleep
    {
        If (_OSI ("Darwin"))
        {
            \_SB.PCI0.RP01.PEGP._ON ()
        }

        XPTS (Arg0)
    }
}

